// Copyright ® 2024 OneSpan North America, Inc. All rights reserved. 

 
/////////////////////////////////////////////////////////////////////////////
//
//
// This file is example source code. It is provided for your information and
// assistance. See your licence agreement for details and the terms and
// conditions of the licence which governs the use of the source code. By using
// such source code you will be accepting these terms and conditions. If you do
// not wish to accept these terms and conditions, DO NOT OPEN THE FILE OR USE
// THE SOURCE CODE.
//
// Note that there is NO WARRANTY.
//
//////////////////////////////////////////////////////////////////////////////


import SwiftUI
import MSSImageScanner
import CoreMedia

/// This sample class presents usage of custom scanner view and `decodeImage` SDK method
class CustomScannerSample {
    private weak var resultDelegate: ScanResultDelegate?
    @Binding var isPresented: Bool

    init(resultDelegate: ScanResultDelegate?, isPresented: Binding<Bool>) {
        self.resultDelegate = resultDelegate
        self._isPresented = isPresented
    }

    // MARK: - Scanned Image Processing
    private func processBuffer(_ sampleBuffer: CMSampleBuffer) {
        guard let image = makeUIImage(from: sampleBuffer) else {
            return
        }

        decodeImage(image)
    }

    /// Used to convert `CMSampleBuffer` data from `AVFoundation` to `UIImage` object
    private func makeUIImage(from sampleBuffer: CMSampleBuffer) -> UIImage? {
        guard let cvBuffer = CMSampleBufferGetImageBuffer(sampleBuffer) else {
            return nil
        }

        let ciImage = CIImage(cvPixelBuffer: cvBuffer)
        let context = CIContext(options: nil)

        guard let cgImage = context.createCGImage(ciImage, from: ciImage.extent) else {
            return nil
        }

        return UIImage(cgImage: cgImage)
    }

    /// Actual `UIImage` decoding stage
    private func decodeImage(_ image: UIImage) {
        do {
            let output = try QRCodeScannerSDK.decodeImage(image, codeType: .all)
            DispatchQueue.main.async {
                self.resultDelegate?.didScan(result: output.result, codeType: output.codeType)
                self.isPresented = false
            }
        } catch ScannerError.invalidImage {
            // Code not decoded - try in next frame
        } catch let error {
            resultDelegate?.didReceive(error)
        }
    }
}

extension CustomScannerSample: CustomScanningDelegate {
    func didCaptureBuffer(_ sampleBuffer: CMSampleBuffer) {
        processBuffer(sampleBuffer)
    }

    func didCancel() {
        resultDelegate?.didCancel()
    }

    func didReceive(_ error: Error) {
        resultDelegate?.didReceive(error)
    }
}
