// Copyright ® 2024 OneSpan North America, Inc. All rights reserved. 

 
/////////////////////////////////////////////////////////////////////////////
//
//
// This file is example source code. It is provided for your information and
// assistance. See your licence agreement for details and the terms and
// conditions of the licence which governs the use of the source code. By using
// such source code you will be accepting these terms and conditions. If you do
// not wish to accept these terms and conditions, DO NOT OPEN THE FILE OR USE
// THE SOURCE CODE.
//
// Note that there is NO WARRANTY.
//
//////////////////////////////////////////////////////////////////////////////


import SwiftUI

struct FullScreenCoverCustom<PresentedView: View>: ViewModifier {
    @Binding var isPresented: Bool
    let presentedView: () -> PresentedView

    func body(content: Content) -> some View {
        ZStack {
            content
            
            if isPresented {
                Color.black
                    .edgesIgnoringSafeArea(.all)
                ZStack {
                    presentedView()
                        .frame(maxWidth: .infinity, maxHeight: .infinity)
                        .statusBarHidden(false)
                }
            }
        }
    }
}

extension View {
    func fullScreenCoverCustom<Context: View>(isPresented: Binding<Bool>,
                                              presentedView: @escaping () -> Context) -> some View {
        modifier(FullScreenCoverCustom(isPresented: isPresented,
                                       presentedView: presentedView))
    }
}
