// Copyright ® 2024 OneSpan North America, Inc. All rights reserved. 

 
/////////////////////////////////////////////////////////////////////////////
//
//
// This file is example source code. It is provided for your information and
// assistance. See your licence agreement for details and the terms and
// conditions of the licence which governs the use of the source code. By using
// such source code you will be accepting these terms and conditions. If you do
// not wish to accept these terms and conditions, DO NOT OPEN THE FILE OR USE
// THE SOURCE CODE.
//
// Note that there is NO WARRANTY.
//
//////////////////////////////////////////////////////////////////////////////


import SwiftUI

struct CombineCameraView: View {
    @ObservedObject private var camera = CombineCameraModel()
    
    @Binding var resultDelegate: ScanResultDelegate?
    @Binding var isPresented: Bool
    
    @State private var hasAppeared = false
    
    var body: some View {
        ZStack {           
            ErrorView(error: camera.error)
            
            cameraOverlay
                .background(CameraFrameView(image: camera.frame))
        }
        .onAppear {
            setup()
        }
        .onDisappear {
            camera.stopSession()
        }
    }
    
    func setup() {
        if !hasAppeared {
            camera.startSession()
            
            let scannerDelegate = CustomScannerSample(resultDelegate: resultDelegate,
                                                      isPresented: $isPresented)
            
            camera.scannerDelegate = scannerDelegate
            hasAppeared = true
        }
    }
}

extension CombineCameraView {
    var cameraOverlay: some View {
        ZStack {
            GradienetView()
            
            ErrorView(error: camera.error)
            
            CrosshairView()
            
            VStack {
                CloseButton(imageName: "close_x_large") {
                    closeButtonAction()
                }
                
                Spacer()

                FooterView()
            }
            .edgesIgnoringSafeArea(.all)
        }
    }
    
    func closeButtonAction() {
        resultDelegate?.didCancel()
        isPresented = false
    }
}

struct CombineCameraView_Previews: PreviewProvider {
    static var resultDelegateMock: Binding<ScanResultDelegate?> {
        .constant(nil)
    }

    static var previews: some View {
        CombineCameraView(resultDelegate: resultDelegateMock,
                          isPresented: .constant(true))
    }
}
