// Copyright ® 2024 OneSpan North America, Inc. All rights reserved. 

 
/////////////////////////////////////////////////////////////////////////////
//
//
// This file is example source code. It is provided for your information and
// assistance. See your licence agreement for details and the terms and
// conditions of the licence which governs the use of the source code. By using
// such source code you will be accepting these terms and conditions. If you do
// not wish to accept these terms and conditions, DO NOT OPEN THE FILE OR USE
// THE SOURCE CODE.
//
// Note that there is NO WARRANTY.
//
//////////////////////////////////////////////////////////////////////////////


import SwiftUI
import MSSImageScanner

struct CustomScanner: View {
    @ObservedObject var camera = CustomCameraModel()

    @Binding var resultDelegate: ScanResultDelegate?    
    @Binding var isPresented: Bool
    
    @State private var isLoading = true
    @State private var hasAppeared = false
    
    var body: some View {
        ZStack {
            Color.black
            
            if isLoading {
                ActivityIndicator()
                    .frame(width: 50, height: 50)
                    .foregroundColor(.white)
            } else {
                CameraWrapperView(camera: camera)
            }

            GradienetView()

            CrosshairView()

            VStack {
                CloseButton(imageName: "close_x") {
                    closeButtonAction()
                }

                Spacer()

                FooterView()
            }
        }
        .edgesIgnoringSafeArea(.all)
        .onAppear {
            setup()
        }
        .onDisappear {
            camera.stopSession()
        }
        .onReceive(camera.$isSessionRunning) { isSessionRunning in
            if isSessionRunning {
                isLoading = false
            }
        }
    }
    
    func setup() {
        if !hasAppeared {
            let scannerDelegate = CustomScannerSample(resultDelegate: resultDelegate,
                                                      isPresented: $isPresented)
            camera.scannerDelegate = scannerDelegate
            hasAppeared = true            
        }
    }
    
    func closeButtonAction() {
        resultDelegate?.didCancel()
        isPresented = false
    }
}

// MARK: - Preview
struct CustomScannerView_Previews: PreviewProvider {
    static var resultDelegateMock: Binding<ScanResultDelegate?> {
        .constant(nil)
    }
    
    static var previews: some View {
        CustomScanner(resultDelegate: resultDelegateMock,
                      isPresented: .constant(true))
    }
}
